/**
 *
 */

package org.objectweb.petals.binding.filetransferbc.listeners;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.objectweb.petals.binding.filetransferbc.FileTransferBCException;
import org.objectweb.petals.binding.filetransferbc.ServiceUnitExtensionsAnalyzer;
import org.objectweb.petals.common.descriptor.Extensions;
import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.bc.AbstractBindingComponent;
import org.objectweb.petals.component.common.bc.ExternalListenerManager;

/**
 * The external listener manager for file transfer BC.
 *
 * @author chamerling - eBM WebSourcing
 *
 */
public class FileTransferBCExternalListenerManager implements
        ExternalListenerManager {

    /**
     * The logger
     */
    private Logger logger;

    /**
     * The component
     */
    private AbstractBindingComponent component;

    /**
     * A map of listeners, key is the address.
     */
    private Map<String, FileTransferBCListener> listeners;

    /**
     * Creates a new instance of {@link FileTransferBCExternalListenerManager}
     *
     * @param logger
     * @param component
     */
    public FileTransferBCExternalListenerManager(Logger logger,
            AbstractBindingComponent component) {
        this.logger = logger;
        this.component = component;
        this.listeners = new HashMap<String, FileTransferBCListener>();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.component.common.bc.ExternalListenerManager#startListening(java.lang.String)
     */
    public void startListening(String address, Extensions extensions)
            throws PEtALSComponentSDKException {
        logger.info("Starting polling directory '" + address + "'");

        long pollingPeriod = ServiceUnitExtensionsAnalyzer.getInstance()
                .retrievePollingPeriod(extensions);

        // Create and register the listener
        FileTransferBCListener listener = null;
        if (!listeners.containsKey(address)) {
            listener = new FileTransferBCListener(logger, component, address,
                    pollingPeriod);
            try {
                listener.init();
            } catch (FileTransferBCException e) {
                throw new PEtALSComponentSDKException(e);
            }
            listeners.put(address, listener);
        } else {
            throw new PEtALSComponentSDKException(
                    "A file transfer listener is already started for the following address : "
                            + address);
        }

        // Start listening for new files to transfer. Scanning period is set to
        // 2 seconds.
        listener.startProcessing();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.component.common.bc.ExternalListenerManager#stopListening(java.lang.String)
     */
    public void stopListening(String address) {
        logger.log(Level.FINE, "Stopping listening on address " + address);
        FileTransferBCListener listener = listeners.get(address);
        if (listener != null) {
            listener.stopProcessing();
        }
        listeners.remove(address);
    }
}
